@@ -1,7 +1,7 @@
-from .core import Primitive, synchronized, PyThread, gated, TimerThread, Timeout
+from .core import Primitive, synchronized, PyThread, gated, Timeout, Timer
 from .dequeue import DEQueue
 from .task_queue import TaskQueue, Task
-from .Scheduler import Scheduler
+from .Scheduler import Scheduler, schedule_job
 from .Subprocess import Subprocess, ShellCommand
 from .RWLock import RWLock
 from .Version import Version
@@ -9,6 +9,7 @@ from .promise import Promise
 from .processor import Processor
 from .flag import Flag
 from .gate import Gate
+from .LogFile import LogFile, LogStream
 from .producer import Producer
 from .escrow import Escrow
 
@@ -30,7 +31,7 @@ __a_ll__ = [
     'Version', '__version__', 'version_info',
     'Timeout',
     'Promise',
-    'Scheduler', 'Processor',
+    'Scheduler',
     'Gate', 'LogFile', 'LogStream',
     'Escrow', 'Producer'
 ]
