@@ -14,6 +14,11 @@ class DEQueue(Primitive):
         self.Closed = True
         self.wakeup()
 
+    @synchronized
+    def open(self):
+        self.Closed = False
+        self.wakeup()
+
     def _wait_for_room(self, timeout):
         # must be called from a synchronized method !
         t0 = time.time()
@@ -40,6 +45,8 @@ class DEQueue(Primitive):
     def __lshift__(self, item):
         return self.append(item)
     
+    put = append
+    
     @synchronized    
     def insert(self, item, timeout=None, force=False):
         if not force:
@@ -51,7 +58,7 @@ class DEQueue(Primitive):
 
     def __rrshift__(self, item):
         return self.insert(item)
-        
+
     @synchronized
     def pop(self, index=0, timeout=None):
         while not (self.List or self.Closed):
@@ -63,6 +70,8 @@ class DEQueue(Primitive):
             item = None         # closed
         return item
 
+    get = pop
+
     #
     # Iterator protocol
     # 
