@@ -91,7 +91,7 @@ class FunctionTask(Task):
         self.F = self.Params = self.Args = None
         return result
         
-class TaskQueue(PyThread):
+class TaskQueue(Primitive):
     
     class ExecutorThread(PyThread):
         def __init__(self, queue, task):
@@ -124,8 +124,8 @@ class TaskQueue(PyThread):
                 self.Queue = None
                 task._Private.Promise = None
 
-    def __init__(self, nworkers=None, capacity=None, stagger=None, tasks = [], delegate=None, 
-                        name=None, daemon=True, start_immediately=True):
+    def __init__(self, nworkers=None, capacity=None, stagger=0.0, tasks = [], delegate=None, 
+                        name=None):
         """Initializes the TaskQueue object
         
         Args:
@@ -134,7 +134,7 @@ class TaskQueue(PyThread):
                         will block. Default: no limit.
 
         Keyword Arguments:
-            stagger (int or float): time interval in seconds between consecutive task starts. None or 0 means no staggering.
+            stagger (int or float): time interval in seconds between consecutive task starts. Default=0, no staggering.
             tasks (list of Task objects): initial task list to be added to the queue
             delegate (object): an object to receive callbacks with task status updates. If None, updates will not be sent.
             name (string): PyThreader object name
@@ -144,48 +144,46 @@ class TaskQueue(PyThread):
             project_attributes (dict): attriutes to attach to the new project
             query (str): query used to create the file list, optional. If specified, the query string will be added to the project as the attribute.
         """
-        PyThread.__init__(self, name=name, daemon=daemon)
+        Primitive.__init__(self, name=name)
         self.NWorkers = nworkers
         self.Threads = []
         self.Queue = DEQueue(capacity)
         self.Held = False
-        self.Stagger = stagger or 0.0
+        self.Stagger = stagger
         self.LastStart = 0.0
         self.StartTimer = None
         self.Delegate = delegate
         for t in tasks:
             self.addTask(t)
         self.Stop = False
-        if start_immediately:
-            self.start()
         
     def stop(self):
         """Stops the queue thread"""
         self.Stop = True
-        self.wakeup()
+        self.Queue.close()
+        self.cancel_alarm()
         
-    def make_task(self, task_or_callable, *params, **args):
-        if isinstance(task_or_callable, Task):
-            # params and args are ignored
-            task = task_or_callable
-        elif callable(task_or_callable):
-            task = FunctionTask(task_or_callable, *params, **args)
-        else:
-            raise ArgumentError("The task argument must be either a callable or a Task subclass instance")
-        return task
-            
     def __add(self, mode, task, *params,
             timeout=None, promise_data=None, after=None, force=False, **args):
-        task = self.make_task(task, *params, **args)
+        if not isinstance(task, Task):
+            if callable(task):
+                task = FunctionTask(task, *params, **args)
+            else:
+                raise ArgumentError("The task argument must be either a callable or a Task subclass instance")
         timeout = timeout.total_seconds() if isinstance(timeout, timedelta) else timeout
+        if isinstance(after, timedelta):
+            after = after.totalseconds()
+        if after is not None and after < 10*365*24*3600:            # ~ Jan 1 1980
+            after = time.time() + after
         task._Private.After = after.timestamp() if isinstance(after, datetime) else after
-        if mode == "insert":
-            self.Queue.insert(task, timeout = timeout, force=force)
-        else:           # mode == "append"
-            self.Queue.append(task, timeout = timeout, force=force)
-        task._queued()
         task._Private.Promise = promise = Promise(data=promise_data)
-        self.wakeup()
+        with self:
+            if mode == "insert":
+                self.Queue.insert(task, timeout = timeout, force=force)
+            else:           # mode == "append"
+                self.Queue.append(task, timeout = timeout, force=force)
+        task._queued()
+        self.start_tasks()
         return promise
 
     def append(self, task, *params, timeout=None, promise_data=None, after=None, force=False, **args):
@@ -197,7 +195,9 @@ class TaskQueue(PyThread):
         Keyword Arguments:
             timeout (int or float or timedelta): time to block if the queue is at or above the capacity. Default: block indefinitely.
             promise_data (object): data to be associated with the task's promise
-            after (int or float or datetime): time to start the task after. Default: start as soon as possible
+            after (int or float or datetime): time to start the task after. 
+                If ``after`` is numeric and < 365 days or it is a datetime.timedelta object, it is interpreted as time relative to the current time.
+                Default: start as soon as possible
             force (boolean): ignore the queue capacity and append the task immediately. Default: False
         
         Returns:
@@ -226,6 +226,8 @@ class TaskQueue(PyThread):
             timeout (int or float or timedelta): time to block if the queue is at or above the capacity. Default: block indefinitely.
             promise_data (object): data to be associated with the task's promise
             after (int or float or datetime): time to start the task after. Default: start as soon as possible
+                If ``after`` is numeric and < 365 days or it is a datetime.timedelta object, it is interpreted as time relative to the current time.
+                Default: start as soon as possible
             force (boolean): ignore the queue capacity and append the task immediately. Default: False
         
         Returns:
@@ -247,19 +249,20 @@ class TaskQueue(PyThread):
         """
         return self.append(task)
 
-    def run(self):
-        while not self.Stop:
-            with self:
-                now = time.time()
-                if self.Stagger is not None and self.LastStart + self.Stagger > now:
-                    self.sleep(self.LastStart + self.Stagger - now)
-                elif self.Queue \
-                            and (self.NWorkers is None or len(self.Threads) < self.NWorkers) \
-                            and not self.Held:
-                    queued = self.Queue.items()
+    @synchronized
+    def start_tasks(self):
+        again = True
+        while not self.Held and not self.Stop and again:
+            again = False
+            now = time.time()
+            if self.Stagger is not None and self.LastStart + self.Stagger > now:
+                self.alarm(self.start_tasks, t = self.LastStart + self.Stagger)
+            elif self.Queue:
+                nrunning = len(self.Threads)
+                if (self.NWorkers is None or nrunning < self.NWorkers):
                     next_task = None
                     sleep_until = None
-                    for t in queued:
+                    for t in self.Queue.items():
                         after = t._Private.After
                         if after is None or after <= now:
                             next_task = t
@@ -275,12 +278,9 @@ class TaskQueue(PyThread):
                         self.LastStart = time.time()
                         t.start()
                         self.call_delegate("taskStarted", self, next_task)
-                    else:
-                        now = time.time()
-                        if now < sleep_until:
-                            self.sleep(sleep_until - now)
-                else:
-                    self.sleep(10)
+                        again = True
+                    elif sleep_until is not None:
+                        self.alarm(self.start_tasks, t=sleep_until)
 
     @synchronized
     def threadEnded(self, t):
@@ -291,7 +291,7 @@ class TaskQueue(PyThread):
         if task.Resubmit:
             after = None if task.ResubmitInterval is None else task.Queued + task.ResubmitInterval
             self.add(task, after=after, force=True)
-        self.wakeup()
+        self.start_tasks()
         
     def call_delegate(self, cb, *params):
         if self.Delegate is not None and hasattr(self.Delegate, cb):
@@ -304,11 +304,7 @@ class TaskQueue(PyThread):
         return self.call_delegate("taskEnded", self, task, result)
         
     def taskFailed(self, task, exc_type, exc_value, tb):
-        if self.Delegate is None:
-            print("Exception in task:", task)
-            traceback.print_exception(exc_type, exc_value, tb, file=sys.stderr)
-        else:
-            return self.call_delegate("taskFailed", self, task,  exc_type, exc_value, tb)
+        return self.call_delegate("taskFailed", self, task,  exc_type, exc_value, tb)
             
     @synchronized
     def waitingTasks(self):
@@ -369,7 +365,7 @@ class TaskQueue(PyThread):
         Releses the queue, allowing new tasks to start
         """        
         self.Held = False
-        self.wakeup()
+        self.start_tasks()
         
     @synchronized
     def is_empty(self):
@@ -406,8 +402,29 @@ class TaskQueue(PyThread):
         Discards all waiting tasks
         """
         self.Queue.flush()
-        self.wakeup()
-            
+
+    @synchronized
+    def cancel(self, task):
+        """
+        Cancel a queued task. Can be used only of the task was added as a Task object. The Promise associated with the Task will be
+        fulfilled with None as the result. If the queue has a delegate, the taskCancelled delegate's method will be called.
+        If the task was already runnig or was not found in the queue, ValueError exception will be raised.
+
+        Args:
+            task (Task): A Task subclass instance
+        
+        Returns:
+            Task: cancelled task
+        """
+
+        try:    self.Queue.remove(task)
+        except ValueError:
+            raise ValueError("Task not in the queue")
+        task._Private.Promise.complete()
+        self.call_delegate("taskCancelled", self, task)
+        self.start_tasks()
+        return task
+
     def __len__(self):
         """
         Equivalent to TaskQueue.nwaiting()
