@@ -7,28 +7,23 @@ done = Flag()
 class MyProcessor(Processor):
     
     def __init__(self, name, nworkers, output = None):
-        self.Name = name
-        Processor.__init__(self, nworkers, output = output)
-        
+        Processor.__init__(self, nworkers, output = output, name=name)
+
     def process(self, lst):
-        print(self.Name, lst)
         lst.append(self.Name)
         if random.random() < 0.2:
             done.value = "done"
-            print("done")
             return None
         else:
             return lst
-        
-        
 
 p1 = MyProcessor("A", 2)
 p2 = MyProcessor("B", 2, output = p1)
 p1.Output = p2
 
 lst = []
-p1.add(lst)
+p1.put(lst)
 
-done.sleep(lambda x: x == "done")
+done.sleep_until("done")
 
-print("--->", lst)
\ No newline at end of file
+print("Result:", lst)
\ No newline at end of file
